package POWER::Utils;

use version; $VERSION = qv('0.0.2');

use warnings;
use strict;
use Carp;

use Perl6::Export::Attrs;


sub clock_monotonic :Export(:DEFAULT);
use Inline C => Config => LIBS => '-lrt';
use Inline C => <<'END_OF_C';
#define __NR_timer_create   259
#define __NR_clock_gettime  (__NR_timer_create+6)
#define SYS_clock_gettime   __NR_clock_gettime
#include <time.h>
#include <sys/syscall.h>
#include <unistd.h>
double clock_monotonic() {
    struct timespec ts;
    // use syscall because glibc-2.3.3 doesn't support CLOCK_MONOTONIC yet
    if (syscall(SYS_clock_gettime, CLOCK_MONOTONIC, &ts) == -1)
        croak("clock_gettime: %s\n", strerror(errno));
    return ts.tv_sec+ts.tv_nsec/1e9;
}
END_OF_C


my $STDREF = qr{SCALAR|ARRAY|HASH|CODE|REF|GLOB|LVALUE};
sub run_callback :Export(:DEFAULT) {
    croak  q{usage: run_callback( CB [, METHOD [, @ARGS]] )}    if @_ < 1;
    my ($cb, $method) = (shift, shift);
    my $cb_type 
        = !ref($cb)                         ? 'CLASS' 
        : ref($cb) eq 'CODE'                ? 'CODE'
        : ref($cb) !~ m{\A$STDREF\z}xmso    ? 'OBJECT'
        :                                     undef
        ;
    if ($cb_type eq 'CLASS' || $cb_type eq 'OBJECT') {
        $cb->$method(@_);
    }
    elsif ($cb_type eq 'CODE') {
        $cb->(@_);
    }
    else {
        croak qq{run_callback: wrong CB $cb};
    }
    return;
}


# from "Perl Cookbook"
my @RandCh = ( 'A'..'Z', 'a'..'z', 0..9, '_' );
sub RandCh :Export(:DEFAULT) {
    croak 'usage: RandCh( LENGTH [, CHARS] )' if !@_;
    my $length = shift;
    croak 'RandCh: LENGTH must be positive' if !defined $length || $length <= 0;
    my @chars  = @_ ? @_ : @RandCh;
    return join("", @chars[ map { rand @chars } 1 .. $length ]);
}


sub Cat :Export(:DEFAULT) {
    croak 'usage: Cat( FILENAME )' if @_ != 1;
    my ($filename) = @_;
    open my $f, '<', $filename or croak "open: $!";
    local $/ if !wantarray;
    return <$f>;
}


sub Echo :Export(:DEFAULT) {
    croak 'usage: Echo( MODE_FILENAME, LIST )' if @_ < 2;
    my $filename = shift;
    open my $f, $filename or croak "open: $!";
    croak 'Echo: Bad file descriptor' if !defined syswrite($f, q{});    # XXX
    @_ = map {"$_\n"} @_ if "@_" !~ /\n/;
    print {$f} join(q{}, @_);
    return;
}


sub Chomp :Export(:DEFAULT) {
    croak 'usage: Chomp( LIST )' if !@_;
    my @lines = @_;
    chomp @lines;
    s/\r\z//xms for @lines;
    return wantarray ? @lines : $lines[0];
}


# TODO     -   ( 'filtered eval.pl :-)')
#         .
sub Eval :Export(:DEFAULT) {
    croak 'usage: Eval( SCALAR [, EVALNAME] )' if !(1 <= @_ && @_ <= 2);
    my ($code, $eval_name) = @_;
    $eval_name ||= '[use Eval(SCALAR, EVALNAME) to see EVALNAME here]';
    open my $f, '<', \$code or croak "open: $!";
    local @INC = ( sub {shift @INC; $f}, @INC );
    do $eval_name;
}


my %Enc = (
    qw( & &amp;   > &gt;   < &lt;   " &quot; ),
    map {chr, "&#$_;"} 0 .. 8, 11 .. 31, 127, 139, 155,
);
sub Enc :Export(:DEFAULT) {
    croak 'usage: Enc( SCALAR )' if @_ != 1;
    my ($html) = @_;
    croak 'Enc: $_[0] undefined' if !defined $html;
    $html =~ s/([\&\>\<\"\0-\10\13-\37\177])/$Enc{$1}/gs;
    return $html;
}


# FIXME   Enc/UnEnc       defined,
#          ,    (..  ref,  ).
my %UnEnc = reverse 
    qw( & &amp;   > &gt;   < &lt;   " &quot; ),
    map {chr, "&#$_;"} 0 .. 255
    ;
sub UnEnc :Export(:DEFAULT) {
    croak 'usage: UnEnc( SCALAR )' if @_ != 1;
    my ($html) = @_;
    croak 'UnEnc: $_[0] undefined' if !defined $html;
    $html =~ s/(&(?:#\d+|\w+);)/$UnEnc{$1}/gs;
    return $html;
}


my %EncUri = map { chr($_), sprintf("%%%02X",$_) } 0..255;
sub EncUri :Export(:DEFAULT) {
    croak 'usage: EncUri( SCALAR [, REGEXP] )' if !(1 <= @_ && @_ <= 2);
    my ($to_escape, $regexp) = @_;
    croak 'EncUri: $_[0] undefined' if !defined $to_escape;
    $regexp ||= qr/[^\w *\@.-]/; 
    $to_escape =~ s/($regexp)/$EncUri{$1}/g;
    $to_escape =~ tr/ /+/ if ! $_[1] or " " =~ /$regexp/;   # XXX
    return $to_escape;
}


my %UnEncUri = map { sprintf("%%%02X",$_), chr($_) } 0..255;
sub UnEncUri :Export(:DEFAULT) {
    croak 'usage: UnEncUri( SCALAR )' if @_ != 1;
    my ($to_unescape) = @_;
    croak 'UnEncUri: $_[0] undefined' if !defined $to_unescape;
    $to_unescape =~ tr/+/ /;
    $to_unescape =~ s/(%[0-9A-Fa-f]{2})/$UnEncUri{uc($1)}/g;
    return $to_unescape;
}


sub UpdateHash :Export { # XXX
    my ($CONFIG, $config) = @_;
    $CONFIG = {} unless ref($CONFIG) eq "HASH";
    $config = {} unless ref($config) eq "HASH";
    for my $param (keys %$config) {
        if (ref($config->{$param}) eq "HASH") {
            $CONFIG->{$param} = {} if ref($CONFIG->{$param}) ne "HASH";
            UpdateHash($CONFIG->{$param}, $config->{$param});
        } else {
            $CONFIG->{$param} = $config->{$param};
        }
    }
    return $CONFIG;
}


1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Utils - Bundle of useful subroutines.


=head1 VERSION

This document describes POWER::Utils version 0.0.1


=head1 SYNOPSIS

 use POWER::Utils;

 $sec = clock_monotonic();

 run_callback({ cb => \&sub });
 run_callback({ cb => 'CLASS', method => 'some_method' });
 run_callback({ cb	=> $obj,
		method	=> 'some_method',
		args	=> [$foo,"bar"],
 });

 $passwd = RandCh(8);

 $passwd_list = Cat("/etc/passwd");
 @passwd_list = Chomp(Cat("/etc/passwd"));

 Echo(">>$0.log", "Debug:", "message...") or die $!;

 Eval("use Any::Source::Filter; # filtered code ...");
 Eval("use Any::Source::Filter; # filtered code ...", 'some my code');

 $html_unsafe = "test < & >";
 $html = Enc($html_unsafe); 
 # $html eq "test &lt; &amp; &gt;"
 $text = UnEnc($html);

 $unsafe = "Programming Perl";
 $safe = EncUri($unsafe); 
 # $safe eq "Programming+Perl"
 $safe = EncUri($unsafe, qr/[a-zA-Z]/);
 $decoded = UnEncUri($safe);


 use POWER::Utils qw( :DEFAULT UpdateHash );
 UpdateHash( $main, $update );


=head1 DESCRIPTION

Bundle of general-purpose subroutines.


=head1 EXPORTS

=over

=item :DEFAULT

clock_monotonic()
run_callback()
RandCh()
Cat()
Chomp()
Echo()
Eval()
Enc()
UnEnc()
EncUri()
UnEncUri()


=item UpdateHash


=back

=head1 INTERFACE 

=over

=item B<clock_monotonic>()

Return: clock_gettime(CLOCK_MONOTONIC) - see L<clock_gettime(2)>.


=item B<run_callback>( CB, METHOD, ARGS )

Run callback in Perl6 style (see http://dev.perl.org/perl6/rfc/321.html).

 CB	        REQUIRED. code ref OR object OR class name
 METHOD     REQUIRED. method name for CB
 ARGS	    OPTIONAL. list with params for CB
 NOTE: METHOD required only if CB is object or class name.

Return: nothing.


=item B<RandCh>( LENGTH [, CHARS] )

Return: string filled by LENGTH random characters.
The random characters are taken from ("A".."Z","a".."z",0..9,"_")
or from CHARS list if specified.


=item B<Chomp>( LIST )

Return: LIST after chomp(LIST) and removing "\r" at end of lines.


=item B<Cat>( FILENAME )

Slurp FILENAME.

NOTE: See L<Perl6::Slurp> for more powerful (but slower) file slurping.

Return:
in SCALAR context contents of FILENAME in single scalar;
in LIST context all lines of FILENAME as list.


=item B<Echo>( MODE_FILENAME, LIST )

Output LIST into MODE_FILENAME.

MODE_FILENAME is used as param for open() as is and it should result in opening
file for writing. Examples of MODE_FILENAME:

 '> /tmp/file'
 '>> /tmp/file'
 '| someprog'

If LIST contain "\n" then output LIST as is. Else append "\n" to end of every
element of LIST.

Return: nothing.


=item B<Eval>( STRING [, NAME] )

Use do() instead of eval() to evaluate perl code in STRING to allow using
source filters in that code. If NAME given it will be shown in case of
errors while executing STRING.


=item B<Enc>( STRING )

Return: STRING after replacing unsafe HTML characters by HTML-entities.


=item B<UnEnc>( STRING )

Return: STRING after decoding all HTML entities.


=item B<EncUri>( STRING [, REGEXP] )

Return: STRING after escaping unsafe URI characters.

REGEXP can be used to define unsafe characters in form: qr/[ABC\d\s]/ .
By default unsafe characters are defined as qr/[^\w *\@.-]/ .


=item B<UnEncUri>( STRING )

Return: STRING after unescaping it as URI.


=item B<UpdateHash>( MAIN, UPDATE )

I<Ugly. DEPRECATED. Will be removed in next releases.>

Update hash ref MAIN with data from hash ref UPDATE. Example:

 my $main = {};
 UpdateHash( $main, {
    PARAM1 => "value1",
    PARAM2 => 15,
    } );
 UpdateHash( $main, {
    PARAM2 => 123,
    PARAM3 => {
        DATA1 => "value31",
        DATA2 => "value32",
        },
    } );
 UpdateHash( $main, {
    PARAM3 => {
        DATA2 => "newvalue32",
        },
    } );
 # Now $main contains:
 # {
 #  PARAM1 => "value1",
 #  PARAM2 => 123,
 #  PARAM3 => {
 #      DATA1 => "value31",
 #      DATA2 => "newvalue32",
 #      },
 # }

Return: MAIN.


=back

=head1 DIAGNOSTICS

=head2 EXCEPTIONS

=over

=item C<< clock_gettime: $! >>

clock_monotonic() got error in low-level L<clock_gettime(3)>.

=item C<< usage: run_callback( CB [, METHOD [, @ARGS]] ) >>

run_callback() was executed without params.

=item C<< run_callback: wrong CB ... >>

First param of run_callback() isn't one of:

 CODE ref
 OBJECT
 CLASS name

=item C<< usage: RandCh( LENGTH [, CHARS] ) >>

RandCh() was executed without params.

=item C<< RandCh: LENGTH must be positive >>

First RandCh() param must be a positive number.

=item C<< usage: Cat( FILENAME ) >>

Cat() must be executed with single param.

=item C<< open: $! >>

This exception can happens in Cat() , Echo() and Eval() .
In case of Cat() and Echo() it mean you give them wrong FILENAME param.
In case of Eval() ... maybe you try to evaluate something strange?

=item C<< usage: Echo( MODE_FILENAME, LIST ) >>

Echo() require at least two params.

=item C<< Echo: Bad file descriptor >>

Echo() unable to write into MODE_FILENAME.

=item C<< usage: Chomp( LIST ) >>

Chomp() require at least one param.

=item C<< usage: Eval( SCALAR [, EVALNAME] ) >>

Eval() require one or two params.

=item C<< usage: Enc( SCALAR ) >>
=item C<< Enc: $_[0] undefined >>

Enc() require one param and it should be defined.

=item C<< usage: UnEnc( SCALAR ) >>
=item C<< UnEnc: $_[0] undefined >>

UnEnc() require one param and it should be defined.

=item C<< usage: EncUri( SCALAR [, REGEXP] ) >>

EncUri() require one or two params.

=item C<< EncUri: $_[0] undefined >>

Guess? Yeah, first EncUri() param must be defined.

=item C<< usage: UnEncUri( SCALAR ) >>
=item C<< UnEncUri: $_[0] undefined >>

UnEncUri() require one param and it should be defined.


=back

=head2 WARNINGS

None.

=head1 CONFIGURATION AND ENVIRONMENT

POWER::Utils requires no configuration files or environment variables.


=head1 DEPENDENCIES

 version
 Perl6::Export::Attrs
 Inline


=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-power-utils@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2005, Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
